from cached_property import cached_property
from onegov.core.layout import ChameleonLayout
from onegov.core.static import StaticFile


class Layout(ChameleonLayout):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.request.include('common')

    @cached_property
    def font_awesome_path(self):
        static_file = StaticFile.from_application(
            self.app, 'font-awesome/css/font-awesome.min.css')

        return self.request.link(static_file)

    @cached_property
    def logo_path(self):
        static_file = StaticFile.from_application(self.app, 'logo.svg')

        return self.request.link(static_file)

    @cached_property
    def town_names_path(self):
        static_file = StaticFile.from_application(self.app, 'towns.json')
        return self.request.link(static_file)


class DefaultLayout(Layout):
    pass


class MailLayout(Layout):

    @cached_property
    def base(self):
        return self.template_loader['mail_layout.pt']

    @cached_property
    def macros(self):
        raise NotImplementedError



import morepath

from onegov.core.security import Public
from onegov.onboarding import _
from onegov.onboarding import OnboardingApp
from onegov.onboarding.layout import DefaultLayout
from onegov.onboarding.models import Assistant, DefaultAssistant
from webob.exc import HTTPMethodNotAllowed


@OnboardingApp.view(model=DefaultAssistant, permission=Public)
def view_default_assistant(self, request):
    return morepath.redirect(request.link(self.assistant))


@OnboardingApp.form(model=Assistant, template='step.pt', permission=Public,
                    form=lambda model, request: model.current_step.form)
def view_assistant(self, request, form):

    # usually this is morepath's job, but we're doing some funny things here ;)
    if form is None and request.method == 'POST':
        return HTTPMethodNotAllowed()

    response = self.current_step.handle_view(request, form)

    if isinstance(response, dict):
        assert 'layout' not in response
        response['layout'] = DefaultLayout(self, request)

        if 'form' not in response:
            response['form'] = form

        if self.is_last_step:
            response['button_text'] = _("Launch")
        else:
            response['button_text'] = _("Continue")

    return response



from onegov.onboarding.models.assistant import (
    Assistant,
    DefaultAssistant,
    Step
)
from onegov.onboarding.models.town_assistant import TownAssistant

__all__ = [
    'Assistant',
    'DefaultAssistant',
    'Step',
    'TownAssistant'
]



import inspect
import time


class Assistant(object):
    """ Describes an assistant guiding a user through onboarding. """

    def __init__(self, app, current_step_number=1):

        self.app = app

        methods = (fn[1] for fn in inspect.getmembers(self))
        methods = (fn for fn in methods if inspect.ismethod(fn))
        methods = (fn for fn in methods if hasattr(fn, 'is_step'))

        self.steps = [Step(fn, fn.order, fn.form) for fn in methods]
        self.steps.sort()

        if current_step_number < 1:
            raise KeyError("Invalid current step")

        if current_step_number > len(self.steps):
            raise KeyError("Invalid current step")

        self.current_step_number = current_step_number

    @property
    def current_step(self):
        return self.steps[self.current_step_number - 1]

    @property
    def progress(self):
        return self.current_step_number, len(self.steps)

    @property
    def is_first_step(self):
        return self.current_step_number == 1

    @property
    def is_last_step(self):
        return self.current_step_number == len(self.steps)

    def for_next_step(self):
        assert not self.is_last_step
        return self.__class__(self.app, self.current_step_number + 1)

    def for_prev_step(self):
        assert not self.is_first_step
        return self.__class__(self.app, self.current_step_number - 1)

    def for_first_step(self):
        return self.__class__(self.app, 1)

    @classmethod
    def step(cls, form=None):
        def decorator(fn):
            fn.is_step = True
            fn.order = time.process_time()
            fn.form = form

            return fn

        return decorator


class Step(object):
    """ Describes a step in an assistant. """

    def __init__(self, view_handler, order, form):
        self.view_handler = view_handler
        self.order = order
        self.form = form

    def __lt__(self, other):
        return self.order < other.order

    def handle_view(self, request, form):
        if form is None:
            return self.view_handler(request)
        else:
            return self.view_handler(request, form)


class DefaultAssistant(object):
    def __init__(self, assistant):
        self.assistant = assistant



import onegov.core
import onegov.onboarding
import os.path
import pytest

from morepath import setup
from onegov.core.utils import scan_morepath_modules
from uuid import uuid4


@pytest.yield_fixture(scope="function")
def onboarding_app(postgres_dsn, temporary_directory, smtp):

    config = setup()
    scan_morepath_modules(onegov.onboarding.OnboardingApp, config)
    config.commit()

    app = onegov.onboarding.OnboardingApp()
    app.namespace = 'test_' + uuid4().hex
    app.configure_application(
        dsn=postgres_dsn,
        filestorage='fs.osfs.OSFS',
        filestorage_options={
            'root_path': os.path.join(temporary_directory, 'file-storage'),
            'create': True
        },
        identity_secure=False,
        disable_memcached=True,
        onboarding={
            'onegov.town': {
                'namespace': 'town_' + uuid4().hex,
                'domain': 'example.org'
            }
        },
    )
    app.set_application_id(app.namespace + '/' + 'test')

    app.mail_host, app.mail_port = smtp.address
    app.mail_sender = 'mails@govikon.ch'
    app.mail_force_tls = False
    app.mail_username = None
    app.mail_password = None
    app.mail_use_directory = False
    app.smtp = smtp

    yield app
